from datetime import date
from django.db import models
from django.core.validators import RegexValidator
from django.forms import ValidationError
from django.contrib.auth.models import AbstractUser

def validate_age(value):
    """Ensure the user is at least 12 years old."""
    today = date.today()
    age = today.year - value.year - ((today.month, today.day) < (value.month, value.day))
    if age < 12:
        raise ValidationError("User must be at least 12 years old.")

class CustomUser(AbstractUser):
    ROLE_CHOICES = [
        ('warga', 'Warga'),
        ('pegawai', 'Pegawai'),
    ]
    username = models.CharField(
        unique=True,
        blank=False,
        max_length=255,
        validators=[
            RegexValidator(
                regex=r'^[A-Za-z0-9._-]+$',
                message='username hanya boleh berisi huruf, angka, dan karakter (".", "_", "-").',
            ),
        ],
        help_text='username hanya boleh berisi huruf, angka, dan karakter (".", "_", "-").'
    )
    role = models.CharField(
        max_length=10,
        choices=ROLE_CHOICES,
        default='warga',
        help_text="Pilih role pengguna (warga atau pegawai)."
    )
    password = models.CharField(
        max_length=255,
        blank=False,
        validators=[
            RegexValidator(
                regex=r'.{8,}',
                message="Password minimal 8 karakter, boleh mengandung huruf, angka, dan karakter spesial."
            )
        ],
        help_text="Password minimal 8 karakter, boleh mengandung huruf, angka, dan karakter spesial."
    )
    tanggal_lahir = models.DateField(
        validators=[validate_age],
        help_text="User harus berusia minimal 12 tahun."
    )
    nomor_hp = models.CharField(
        max_length=15,
        blank=False,
        validators=[
            RegexValidator(
                regex=r'^62\d{7,13}$',
                message="Nomor HP harus terdiri dari 8 hingga 15 digit tanpa spasi atau karakter lain."
            )
        ],
        help_text="Kode negara dan nomor telepon tanpa '+' atau '-' (contoh: 62123456)."
    )
    email = models.CharField(
        unique=True,
        max_length=255,
        blank=False,
        validators=[
            RegexValidator(
                regex=r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$',
                message="Masukkan email yang valid."
            )
        ],
        help_text="Email sesuai dengan format email valid."
    )
    url_blog = models.CharField(
        max_length=255,
        blank=False,
        validators=[
            RegexValidator(
                regex=r'^https:\/\/([\da-z\.-]+)\.([a-z]{2,6})([\/\w\.-]*)*\/?$',
                message="Masukkan URL blog yang valid dan secure."
            )
        ],
        help_text="URL Blog harus valid dan diawali dengan https://"
    )
    deskripsi_diri = models.TextField(
        validators=[
            RegexValidator(
                regex=r'^.{5,1000}$',
                message="Deskripsi Diri antara 5 hingga 1000 karakter."
            )
        ],
        help_text="Deskripsi Diri antara 5 hingga 1000 karakter."
    )
    npwp = models.CharField(
        max_length=20,
        blank=False,
        validators=[
            RegexValidator(
                regex=r'^\d{2}\.\d{3}\.\d{3}\.\d{1}-\d{3}\.\d{3}$',
                message="NPWP harus memiliki format XX.XXX.XXX.X-XXX.XXX dimana X adalah angka."
            )
        ],
        help_text="Contoh format: 12.345.678.9-012.345"
    )
    no_kk = models.CharField(
        max_length=16,
        blank=False,
        validators=[
            RegexValidator(
                regex=r'^\d{16}$',
                message="No. KK harus terdiri dari 16 digit."
            )
        ],
        help_text="Masukkan No. KK yang valid, 16 digit angka."
    )

    REQUIRED_FIELDS = ['email', 'tanggal_lahir', 'nomor_hp', 'url_blog', 'deskripsi_diri', 'npwp', 'no_kk', 'role']

class PermitType(models.Model):
    """Model to represent a permit type."""
    name = models.CharField(max_length=255, unique=True)

    def __str__(self):
        return self.name

class PermitApplication(models.Model):
    """Model to represent a submitted permit application."""
    STATUS_CHOICES = [
        ('PENDING', 'Pending'),
        ('APPROVED', 'Approved'),
        ('REJECTED', 'Rejected'),
    ]
    file = models.FileField(upload_to='permits/')
    time_submitted = models.DateTimeField(auto_now_add=True)
    status = models.CharField(max_length=255, choices=STATUS_CHOICES, default='PENDING')
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE, related_name='permit_applications')
    permit_type = models.ForeignKey(PermitType, on_delete=models.CASCADE)

    def __str__(self):
        return f"{self.user.username} - {self.permit_type.name} - {self.status}"

class ApprovalHistory(models.Model):
    """Model to represent the approval history of a permit application."""
    time_approved = models.DateTimeField(auto_now_add=True)
    officer = models.ForeignKey(CustomUser, on_delete=models.CASCADE)
    permit = models.OneToOneField(PermitApplication, on_delete=models.CASCADE, related_name='approval_history')

    def __str__(self):
        return f"{self.permit} approved by {self.officer.username} at {self.time_approved}"