from django.contrib.auth.decorators import login_required, user_passes_test
from django.contrib.auth.forms import AuthenticationForm
from django.contrib.auth import login, logout
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render, redirect, get_object_or_404
from django.contrib import messages
from django.db import transaction
from django.core.exceptions import PermissionDenied

from main.models import CustomUser, PermitApplication, PermitType, ApprovalHistory
from .forms import UserForm, PermitApplicationForm

def is_pegawai(user):
    return user.is_authenticated and user.role == 'pegawai'

def is_warga(user):
    return user.is_authenticated and user.role == 'warga'

@transaction.atomic
def register(request):
    if request.method == 'POST':
        form = UserForm(request.POST)
        if form.is_valid():
            user = form.save(commit=False)
            # Set the password properly to hash it.
            user.set_password(form.cleaned_data.get('password'))
            user.save()
            formatted_data = [
                {'label': str(field.label), 'value': str(form.cleaned_data[field.name])}
                for field in form
            ]
            request.session['form_data'] = formatted_data
            return redirect('main:login')
        else:
            messages.error(request, "Registration failed. Please correct the errors.")
    else:
        form = UserForm()
    return render(request, 'register.html', {'form': form})

def login_user(request):
    if request.method == 'POST':
        form = AuthenticationForm(data=request.POST)
        if form.is_valid():
            user = form.get_user()
            login(request, user)
            if user.role == 'warga':
                return redirect("main:warga_main")
            else:
                return redirect("main:pegawai_main")
        else:
            messages.error(request, "Invalid credentials, please try again.")
    else:
        form = AuthenticationForm()
    return render(request, 'login.html', {'form': form})

def logout_user(request):
    logout(request)
    return redirect("main:login")

@login_required(login_url='main:login')
@user_passes_test(is_warga, login_url='main:login')
def warga_main(request):
    permit_applications = PermitApplication.objects.filter(user=request.user).order_by('-time_submitted')
    return render(request, 'warga_main.html', {'permit_applications': permit_applications})

@login_required(login_url='main:login')
@user_passes_test(is_pegawai, login_url='main:login')
def pegawai_main(request):
    permit_applications = PermitApplication.objects.filter(status='PENDING').order_by('time_submitted')
    return render(request, 'pegawai_main.html', {'permit_applications': permit_applications})

@login_required(login_url='main:login')
@user_passes_test(is_pegawai, login_url='main:login')
def permit_history(request):
    permits = PermitApplication.objects.exclude(status='PENDING').order_by('-time_submitted').select_related('user')
    return render(request, 'permit_history.html', {'permits': permits})

@transaction.atomic
@login_required(login_url='main:login')
@user_passes_test(is_warga, login_url='main:login')
def submit_permit(request):
    if request.method == 'POST':
        form = PermitApplicationForm(request.POST, request.FILES)
        if form.is_valid():
            with transaction.atomic():
                permit_application = form.save(commit=False)
                permit_application.user = request.user
                permit_application.save()
            messages.success(request, "Permit application submitted successfully!")
            return redirect("main:warga_main")
        else:
            messages.error(request, "Permit application failed. Please correct the errors.")
    else:
        form = PermitApplicationForm()
    return render(request, 'submit_permit.html', {'form': form})

@transaction.atomic
@login_required(login_url='main:login')
@user_passes_test(is_pegawai, login_url='main:login')
def approve_permit(request, permit_id):
    permit = get_object_or_404(PermitApplication, pk=permit_id)
    with transaction.atomic():
        permit.status = 'APPROVED'
        permit.save()
        ApprovalHistory.objects.create(officer=request.user, permit=permit)
    messages.success(request, "Permit application approved successfully!")
    return redirect("main:pegawai_main")

@transaction.atomic
@login_required(login_url='main:login')
@user_passes_test(is_pegawai, login_url='main:login')
def reject_permit(request, permit_id):
    permit = get_object_or_404(PermitApplication, pk=permit_id)
    with transaction.atomic():
        permit.status = 'REJECTED'
        permit.save()
        ApprovalHistory.objects.create(officer=request.user, permit=permit)
    messages.success(request, "Permit application rejected successfully!")
    return redirect("main:pegawai_main")

@login_required(login_url='main:login')
def show_info(request):
    data = request.session.pop('form_data', None)
    if not data:
        user = request.user
        data = [
            {'label': 'Username', 'value': user.username},
            {'label': 'Email', 'value': user.email},
            {'label': 'Tanggal Lahir', 'value': user.tanggal_lahir},
            {'label': 'Nomor HP', 'value': user.nomor_hp},
            {'label': 'URL Blog', 'value': user.url_blog},
            {'label': 'Deskripsi Diri', 'value': user.deskripsi_diri},
            {'label': 'NPWP', 'value': user.npwp},
            {'label': 'No. KK', 'value': user.no_kk},
            {'label': 'Role', 'value': user.role},
        ]
    return render(request, 'profile.html', {
        'data': data,
        'title': 'User Profile',
        'show_nav': True
    })

@login_required(login_url='main:login')
@user_passes_test(is_pegawai, login_url='main:login')
def applicant_profile(request, user_id):
    applicant = get_object_or_404(CustomUser, pk=user_id, role='warga')
    data = [
        {'label': 'Username', 'value': applicant.username},
        {'label': 'Email', 'value': applicant.email},
        {'label': 'Tanggal Lahir', 'value': applicant.tanggal_lahir},
        {'label': 'Nomor HP', 'value': applicant.nomor_hp},
        {'label': 'URL Blog', 'value': applicant.url_blog},
        {'label': 'Deskripsi Diri', 'value': applicant.deskripsi_diri},
        {'label': 'NPWP', 'value': applicant.npwp},
        {'label': 'No. KK', 'value': applicant.no_kk},
        {'label': 'Role', 'value': applicant.role},
    ]
    return render(request, 'profile.html', {
        'data': data,
        'title': f'Applicant Profile: {applicant.username}',
        'show_nav': False,
        'back_url': 'main:pegawai_main'
    })